// File:   rl_dielectricpod_rdb.cc
// Author: Terry Gaetz

// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of rl_ray
//
// rl_ray is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// rl_ray is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

// Description: read in dielectric constant information from rdb table; 
//              organize into bsearchable array
//

#include <rl_raysuplib/rl_DielectricPOD_rdb.h>
#include <rl_raylib/rl_DielectricPODArray.h>

//#include <rl_Traits.h>

#include <cstdio>                     // fopen
#include <cstring>
#include <cmath>

#include <tracefctxx/TraceFct.h>
#include <tracefct/exiterrvals.h>
#include <mst_rdb/mst_rdb.h>
#include <suplib/str.h>

using namespace std;

#define NUM_DATA_ITEMS 3
/* enum        EColname     { Eenergy,  Ealpha,  Egamma  }; */
/* static char *colname[] = { "energy", "alpha", "gamma" }; */

typedef enum
{
  Ealpha_gamma
} ERfldataType;

static TokListToken
dieldata_type[] =
{
  { "alpha_gamma",  Ealpha_gamma },
};
TokList dieldata_types = GenTokList( dieldata_type );

//-------------------------------------------------------------------------
// dtor, ctors...

rl_DielectricPOD_rdb::
~rl_DielectricPOD_rdb()
{}

rl_DielectricPOD_rdb::
rl_DielectricPOD_rdb( char const rdb_file[] )
  : rl_DielectricPODArray()
{ if ( strlen(rdb_file) != 0 ) { init( rdb_file ); } }

void rl_DielectricPOD_rdb::
init( char const rdb_filename_in[] )
{
  TraceFct tf( "rl_DielectricPOD_rdb::init" );
  typedef struct rfl_record
  {
    double energy;
    double alpha;
    double gamma;
  } rfl_record;
  rfl_record record;
  RDBFieldStInfo fields[] =
  {
    RDBentry(energy, RDB_Num, rfl_record),
    RDBentry(alpha,  RDB_Num, rfl_record),
    RDBentry(gamma,  RDB_Num, rfl_record)
  };
  #define NFIELDS (sizeof(fields) / sizeof(RDBFieldStInfo) )

  /*
   * open file containing reflectance data
   */

  int error;
  char *rdb_filename =
    str_interp( rdb_filename_in, 1, NULL, NULL, &error );

  if ( error )
    tf.die( "error interpolating filename: %s", rdb_filename_in );

  FILE* in = fopen(rdb_filename, "r");
  if (! in )
    tf.exit(ExitERR_fopen, "unable to open input `%s'", rdb_filename);

  rdbHeader* hdr = rdb_rd_hdr(in);  
  if (! hdr )
    tf.die( "rdb file `%s' contains no rdb headers!", rdb_filename );

  DataColumnMap_st* map = rdb_map_cols_stst(hdr, NFIELDS, fields);

  /*
   * count number of rows (excluding header & comments) in rdb file
   */
  nelts_ = rdb_count(in, hdr);
  if ( nelts_ < 0 )
    tf.die(
       "reflectance rdb file '%s' is not rewindable", rdb_filename);

  else if ( nelts_ < 2 )
    tf.die(
       "reflectance rdb file '%s' must have at least two rows", rdb_filename);


  /*
   * allocate arrays to hold data
   */
  double* energy = new double[nelts_];
  double* alpha  = new double[nelts_];
  double* gamma  = new double[nelts_];

  data_ = new rl_Traits::rl_DielectricPOD[ nelts_ ];
  if (! data_ )
    tf.exit(ExitERR_alloc, "unable to malloc rl_DielectricPOD array");


  /*
   * read reflectance data into rl_DielectricPOD array
   */
  int   n;
  for ( n = 0 ; n < nelts_ ; ++n )
  {
    rdb_col_read_st(in, hdr, map, &record);
    energy[n] = record.energy;
    alpha[n]  = record.alpha;
    gamma[n]  = record.gamma;
  }

  rl_DielectricPODArray::init( nelts_, energy, alpha, gamma );

  /* 
   * clean up
   */
  delete [] energy;
  delete [] alpha;
  delete [] gamma;

  free( rdb_filename );
  rdb_free_map(map);       /* free up data column map */
  rdb_free_hdr(hdr);       /* free up rdbHeader dynamic allocation */
  fclose(in);
}
